package yaml

import (
	"path/filepath"

	"git.perx.ru/perxis/perxis-go/pkg/errors"
	"gopkg.in/yaml.v3"
)

// IncludeResolver включает содержимое целевого YAML-файла в поле.
func IncludeResolver(tp TagProcessor, node *yaml.Node) (*yaml.Node, error) {
	if node.Kind != yaml.ScalarNode {
		return nil, errors.New("!include on a non-scalar node")
	}

	if ext := filepath.Ext(node.Value); ext != ".yaml" && ext != ".yml" {
		return nil, errors.New("!include on file with unknown extension")
	}

	file, err := tp.FS().Open(node.Value)
	if err != nil {
		return nil, err
	}
	defer func() { _ = file.Close() }()

	out := new(yaml.Node)
	err = yaml.NewDecoder(file).Decode(WithTagProcessor(tp.FS())(out))
	if err != nil {
		return nil, err
	}

	return out, nil
}
